import stackless as ss
from PySFML import sf
from settings.settings import *
import globals.globals as g
import channels.channels as channels
from . import channels as bgChannels
import graphics.graphics as graphics
import graphics.animation as animation
from . import collisions as coll
from .types import *
import time

idleAnimDesc = (dataPath + "mochicat/idle/mochiIdle", 0, 1.0)
walkAnimDesc = (dataPath + "mochicat/walk/mochiWalk", 3, 0.1)
jumpAnimDesc = (dataPath + "mochicat/jump/mochiJump", 1, 0.2)
fallAnimDesc = (dataPath + "mochicat/fall/mochiFall", 1, 0.2)
headbuttAnimDesc = (dataPath + "mochicat/headbutt/mochiHeadbutt", 1, 0.2)
hurtAnimDesc = (dataPath + "mochicat/hurt/mochiHurt", 0, 0.2)
dashAnimDesc = (dataPath + "mochicat/dash/mochiDash", 0, 0.4)

walkVel     = 300.0
gravity     = 2500.0
jumpVel     = -900.0
dblJumpVel  = -900.0

teleportDist        = 90.0
teleportCooldown    = 0.25  # seconds

catStartX   = 50.0
catStartY   = 450.0

rectWidth   = 53.0
rectHeight  = 37.0
rectOffsetX = 5.0
rectOffsetY = -5.0

attackRectWidth     = 53.0
attackRectHeight    = 37.0
attackRectOffsetX   = 5.0 + -rectWidth
attackRectOffsetY   = -5.0

headbuttVelX = 500.0
headbuttVelY = -200.0
attackDuration = 0.20    # seconds

headbuttReboundVelX = 200.0
headbuttReboundVelY = -200.0

class Cat:
    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.direction = Direction.RIGHT
        self.rect = self.generateRect(x, y)
        self.velX = 0.0
        self.velY = 0.0
        self.onGround = False
        self.doubleJumped = False
        self.lastAttackTime = time.time()
        self.attacking = False
        self.rebounding = False
        self.lastDashTime = time.time()
        self.lastTeleportTime = time.time()
        self.teleportedInAir = False
        self.dashing = False
        self.dashAnimX = x
        self.dashAnimY = y

        # animations
        self.idleAnim = animation.Animation(idleAnimDesc, True)
        self.walkAnim = animation.Animation(walkAnimDesc, True)
        self.jumpAnim = animation.Animation(jumpAnimDesc, False)
        self.fallAnim = animation.Animation(fallAnimDesc, False)
        self.headbuttAnim = animation.Animation(headbuttAnimDesc, False)
        self.hurtAnim = animation.Animation(hurtAnimDesc, False)
        self.dashAnim = animation.Animation(dashAnimDesc, False)
        self.anim = self.idleAnim

    def draw(self):
        #graphics.drawRect(g.window, self.rect, sf.Color.Magenta)

        if self.attacking:
            #graphics.drawRect(g.window, self.generateAttackRect(self.x, self.y), sf.Color.Red)
            pass

        self.anim.setFlipped(self.direction == Direction.RIGHT)
        self.anim.draw(self.x, self.y)

        if time.time() - self.lastTeleportTime <= teleportCooldown / 2.0:
            self.dashAnim.draw(self.dashAnimX, self.dashAnimY)

    def generateRect(self, x, y):
        return sf.FloatRect(x + rectOffsetX, y + rectOffsetY,
                            x + rectOffsetX + rectWidth,
                            y + rectOffsetY + rectHeight)

    def generateAttackRect(self, x, y):
        if self.direction == Direction.RIGHT:
            return sf.FloatRect(x + rectOffsetX + rectWidth + attackRectOffsetX,
                                y + rectOffsetY + attackRectOffsetY,
                                x + rectOffsetX + rectWidth + attackRectOffsetX + attackRectWidth,
                                y + rectOffsetY + attackRectOffsetY + attackRectHeight)
        else:
            return sf.FloatRect(x + rectOffsetX - attackRectOffsetX - attackRectWidth,
                                y + rectOffsetY + attackRectOffsetY,
                                x + rectOffsetX - attackRectOffsetX,
                                y + rectOffsetY + attackRectOffsetY + attackRectHeight)

    def move(self, x, y):
        self.x = x
        self.y = y
        self.rect = self.generateRect(x, y)

    def canTeleport(self):
        if self.teleportedInAir:
            return False
        else:
            return time.time() - self.lastTeleportTime >= teleportCooldown

    def update(self, inputState):
        timeNow = time.time()

        # walk left/right
        if not self.attacking and not self.dashing:
            self.velX = 0.0
            if inputState.keyDown("left") or inputState.joyLeft:
                self.velX = -walkVel
                self.direction = Direction.LEFT
                self.walkAnim.setFlipped(False)
            elif inputState.keyDown("right") or inputState.joyRight:
                self.velX = walkVel
                self.direction = Direction.RIGHT
                self.walkAnim.setFlipped(True)

        # jump
        jumpPressed = inputState.keyPressed("c") or inputState.joyPressed("jump")
        if jumpPressed and self.onGround:
            self.velY = jumpVel
        # double jump
        elif jumpPressed and not self.doubleJumped:
            self.doubleJumped = True
            self.velY = dblJumpVel

        # teleport
        if (inputState.keyPressed("z") or inputState.joyPressed("dash")) and self.canTeleport():
            self.velX = 0.0
            self.velY = 0.0

            self.dashAnimX = self.x
            self.dashAnimY = self.y
            self.dashAnim.setFlipped(self.direction == Direction.RIGHT)

            if inputState.keyDown("up"):
                self.move(self.x, self.y - teleportDist)
            elif inputState.keyDown("down"):
                self.move(self.x, self.y + teleportDist)
            elif self.direction == Direction.LEFT:
                self.move(self.x - teleportDist, self.y)
            else:
                self.move(self.x + teleportDist, self.y)

            if not self.onGround:
                self.teleportedInAir = True
            self.lastTeleportTime = timeNow

        # attack
        if timeNow - self.lastAttackTime > attackDuration:
            self.attacking = False
            self.rebounding = False
        if inputState.keyPressed("x") or inputState.joyPressed("attack"):
            if timeNow - self.lastAttackTime >= attackDuration:
                self.attacking = True
                self.lastAttackTime = timeNow
                if self.direction == Direction.RIGHT:
                    self.velX = headbuttVelX
                else:
                    self.velX = -headbuttVelX
                self.velY = headbuttVelY

        # gravity
        if not self.dashing:
            self.velY += gravity * g.timeDelta

        # new position
        newX = self.x + self.velX * g.timeDelta
        newY = self.y + self.velY * g.timeDelta

        # prevent cat from teleporting through walls
        if newX < 0.0:
            newX = 0.0
        elif newX > windowWidth - rectWidth:
            newX = windowWidth - rectWidth

        # prevent cat from falling through floor
        if newY > windowHeight:
            newY = catStartY
            self.velY = 0.0

        newRect = self.generateRect(newX, newY)

        self.updateCollision(newX, newY, newRect)

    def updateCollision(self, newX, newY, newRect):
        # submit collision detection requests
        collisionInfoReq = [coll.CollisionInfo("cat", newRect), 
                            coll.CollisionRequest("catReq", newRect, ["floor", "wall", "enemy", "enemyAttack"])]
        if self.attacking:
            collisionInfoReq.append(coll.CollisionInfo("catAttack", self.generateAttackRect(self.x, self.y)))
        bgChannels.collisionsChannel.send(collisionInfoReq)
        collResponses = bgChannels.collisionResponsesChannel.receive()

        self.onGround = False

        # process collisions
        touchedSurface = False
        collResp = collResponses["catReq"]
        for cr in collResp:
            if cr.intersects:

                # touching floor
                if cr.collType == "floor":
                    self.move(newX, newY)
                    self.unstickCollision(newX, newY, newRect, cr.collRect)
                    self.velY = 0.0
                    self.onGround = True
                    self.doubleJumped = False
                    self.teleportedInAir = False

                    touchedSurface = True

                # touching wall
                elif cr.collType == "wall":
                    if not self.onGround:
                        self.move(newX, newY)
                    self.unstickCollision(newX, newY, newRect, cr.collRect)

                    touchedSurface = True

                # touching enemy
                elif cr.collType == "enemy":
                    # attack rebound
                    if self.attacking and not self.rebounding:
                        self.attackRebound()

                # touching enemy attack
                elif cr.collType == "enemyAttack":
                    # hurt
                    if self.anim != self.hurtAnim:
                        self.anim = self.hurtAnim
                        self.anim.reset()

                        bgChannels.scoreChannel.send((-20, True))

        if not touchedSurface:
            self.move(newX, newY)

        self.updateAnimation()
        self.anim.update(g.timeDelta)

    def attackRebound(self):
        if self.direction == Direction.RIGHT:
            self.velX = -headbuttReboundVelX
        else:
            self.velX = headbuttReboundVelX
        self.velY = headbuttReboundVelY

        self.rebounding = True

    def updateAnimation(self):
        if self.anim == self.hurtAnim:
            if not self.hurtAnim.finished:
                return

        if self.attacking:
            if self.anim != self.headbuttAnim:
                self.anim = self.headbuttAnim
                self.anim.reset()
            return

        if self.onGround:
            # idle
            if self.velX == 0.0:
                self.anim = self.idleAnim
            # walk
            else:
                self.anim = self.walkAnim
        else:
            # jump
            if self.velY < 0.0:
                if self.anim != self.jumpAnim:
                    self.anim = self.jumpAnim
                    self.anim.reset()
            # fall
            else:
                if self.anim != self.fallAnim:
                    self.anim = self.fallAnim
                    self.anim.reset()

    def unstickCollision(self, x, y, rect, collRect):
        (transX, transY) = coll.minTranslation(rect, collRect)
        if transY == 0.0:
            self.move(x - transX, self.y)
        else:
            self.move(self.x, y - transY)

    def info(self):
        return CatInfo((self.x, self.y), self.onGround, (rectWidth, rectHeight))

def catTasklet():
    cat = Cat(catStartX, catStartY)

    while 1:
        inputState = channels.inputChannel.receive()

        cat.update(inputState)
        channels.broadcast(bgChannels.catInfoChannel, cat.info())

        cat.draw()
