from PySFML import sf
import channels.channels as channels
import globals.globals as globals
from math import fabs
from . import channels as bgChannels

collisionObjects = dict()
collisionResponses = dict()

class CollisionInfo:
    def __init__(self, collisionType, rect):
        self.collisionType = collisionType
        self.rect = rect

class CollisionRequest:
    def __init__(self, requestId, rect, collisionTypes):
        self.requestId = requestId
        self.rect = rect
        self.collisionTypes = collisionTypes

class CollisionResponse:
    def __init__(self, collRect, intersects, collType):
        self.collRect = collRect
        self.intersects = intersects
        self.collType = collType

def collisionIntersect(collReq):
    collResp = []
    for collType in collReq.collisionTypes:
        if collType in collisionObjects:
            for collObj in collisionObjects[collType]:
                if collReq.rect.Intersects(collObj.rect):
                    collResp.append(CollisionResponse(collObj.rect, True, collType))
    if collResp == []:
        return [CollisionResponse(None, False, None)]
    else:
        return collResp

def collisionsTasklet():
    while 1:
        collisionObjects.clear()
        collisionRequests = []

        bgChannels.runCollisionsChannel.receive()

        # get collision information
        numSenders = 0
        while bgChannels.collisionsChannel.balance > 0:
            collMsgList = bgChannels.collisionsChannel.receive()
            anyCollReqs = False
            for collMsg in collMsgList:
                # collision info
                if type(collMsg) == CollisionInfo:
                    if not collMsg.collisionType in collisionObjects:
                        collisionObjects[collMsg.collisionType] = []
                    collisionObjects[collMsg.collisionType].append(collMsg)
                # collision requests
                elif type(collMsg) == CollisionRequest:
                    anyCollReqs = True
                    collisionRequests.append(collMsg)
            if anyCollReqs:
                numSenders += 1

        collisionResponses.clear()
        for collReq in collisionRequests:
            collResp = collisionIntersect(collReq)
            collisionResponses[collReq.requestId] = collResp

        channels.broadcast(bgChannels.collisionResponsesChannel, collisionResponses, numSenders)

def minTranslation(rect1, rect2):
    left =  rect1.Right - rect2.Left
    right = rect1.Left - rect2.Right
    up = rect1.Top - rect2.Bottom
    down = rect1.Bottom - rect2.Top

    transX =  left if fabs(left) < fabs(right) else right
    transY = up if fabs(up) < fabs(down) else down

    if fabs(transX) < fabs(transY):
        return (transX, 0.0)
    else:
        return (0.0, transY)
