import stackless as ss
from PySFML import sf
from settings.settings import *
import globals.globals as g
import graphics.animation as animation
import channels.channels as channels
from . import channels as bgChannels
import graphics.graphics as graphics
from . import collisions as coll
from .types import *
import time
import math

gravity = 2000.0

start1X   = 50.0
start1Y   = 450.0
start2X   = 900.0
start2Y   = 450.0
start3X   = 600.0
start3Y   = 450.0

wave2Time = 13.0   # secs
wave3Time = 26.0   # secs

walkVel = 80.0

rectWidth   = 83.0
rectHeight  = 37.0
rectOffsetX = 0.0
rectOffsetY = 0.0

projRectWidth   = 33.0
projRectHeight  = 33.0
projRectOffsetX = -17.0
projRectOffsetY = -17.0

initialHealth = 75.0
healthOffsetX = 10.0
healthOffsetY = -50.0
healthUnitWidth = 1.0
healthHeight = 2.0

hitDamage = 25

fireDelay = 2.0

projSlowFactor = 2.0
projRotSpeed = 400.0

bubbleAttackImg = sf.Image()
bubbleAttackImg.LoadFromFile(dataPath + "enemies/fish-tank/bubble-attack.png")
bubbleAttackSpr = sf.Sprite(bubbleAttackImg)

bubbleAttackSpr.SetCenter(23.0, 19.0)

fishTankAnimDesc = (dataPath + "enemies/fish-tank/fish-tank", 2, 0.30)

class Enemy1Proj:
    id = 0

    def __init__(self, x, y, catInfo):
        self.x = catInfo.x
        self.y = 0.0
        self.velX = 0.0
        self.velY = 200.0
        self.rot = 0.0

        self.collReq = "enemy1Proj" + str(Enemy1Proj.id)
        Enemy1Proj.id += 1

    def updatePreColl(self, catInfo):
        timeNow = time.time()

        # new position
        self.x = self.x + self.velX * (g.timeDelta / projSlowFactor)
        self.y = self.y + self.velY *  (g.timeDelta / projSlowFactor)

        self.rot += projRotSpeed * g.timeDelta
        while self.rot > 360.0:
            self.rot -= 360.0

        self.velY += gravity * (g.timeDelta / projSlowFactor)

        self.newRect = self.generateRect(self.x, self.y)

        # return collision detection requests
        return [coll.CollisionInfo("enemyAttack", self.newRect), 
                coll.CollisionRequest(self.collReq, self.newRect, ["cat"])]

    def updatePostColl(self, catInfo, collResponses):
        if self.collReq not in collResponses:
            return

        # process hit collision
        collHitResp = collResponses[self.collReq]
        for chr in collHitResp:
            if chr.intersects:
                if chr.collType == "cat":
                    #print("HIT")
                    pass

    def generateRect(self, x, y):
        return sf.FloatRect(x + projRectOffsetX, y + projRectOffsetX,
                            x + projRectOffsetX + projRectWidth,
                            y + projRectOffsetY + projRectHeight)

    def draw(self):
        #graphics.drawRect(g.window, self.generateRect(self.x, self.y), sf.Color.Green)
        bubbleAttackSpr.SetRotation(self.rot)
        bubbleAttackSpr.SetPosition(self.x, self.y)
        g.window.Draw(bubbleAttackSpr)

    def isDead(self):
        return (self.x < 0.0 or self.x > windowWidth or self.y < 0.0 or self.y > windowHeight)

class Enemy1:
    id = 0

    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.direction = Direction.RIGHT
        self.rect = self.generateRect(x, y)
        self.velX = 0.0
        self.velY = 0.0
        self.health = initialHealth
        self.beingHit = False

        self.anim = animation.Animation(fishTankAnimDesc, True)

        self.lastFireTime = time.time() - fireDelay

        self.collReq = "enemy1Req" + str(Enemy1.id)
        self.collHitReq = "enemy1HitReq" + str(Enemy1.id)
        Enemy1.id += 1

    def isDead(self):
        return self.health <= 0

    def draw(self):
        if self.health > 0:
            healthRect = self.generateHealthRect()
            graphics.drawRect(g.window, healthRect, sf.Color.Red)
        #graphics.drawRect(g.window, self.rect, sf.Color.Blue)

        self.anim.setFlipped(self.direction == Direction.LEFT)
        self.anim.draw(self.x, self.y)

    def generateRect(self, x, y):
        return sf.FloatRect(x + rectOffsetX, y + rectOffsetY,
                            x + rectOffsetX + rectWidth,
                            y + rectOffsetY + rectHeight)

    def generateHealthRect(self):
        return sf.FloatRect(self.x + healthOffsetX, self.y + healthOffsetY,
                            self.x + healthOffsetX + self.health * healthUnitWidth,
                            self.y + healthOffsetY + healthHeight)

    def move(self, x, y):
        self.x = x
        self.y = y
        self.rect = self.generateRect(x, y)

    def runAI(self, catInfo):
        timeNow = time.time()


        if catInfo.onGround:
            if catInfo.x < self.x:
                if self.velX != -walkVel:
                    self.velX = -walkVel
                    self.direction = Direction.LEFT
            else:
                if self.velX != walkVel:
                    self.velX = walkVel
                    self.direction = Direction.RIGHT

        inAttackDist = math.fabs(catInfo.x - self.x) <= 200.0
        if inAttackDist:
            self.velX = 0.0

            if timeNow - self.lastFireTime >= fireDelay:
                bgChannels.projAddChannel.send(Enemy1Proj(self.x, self.y, catInfo))
                self.lastFireTime = timeNow

    def updatePreColl(self, catInfo):
        timeNow = time.time()

        self.velY += gravity * g.timeDelta
        self.runAI(catInfo)

        # new position
        newX = self.x + self.velX * g.timeDelta
        newY = self.y + self.velY * g.timeDelta

        # prevent enemy from falling through floor
        if newY > windowHeight:
            newY = start1Y
            self.velY = 0.0

        self.anim.update(g.timeDelta)

        self.newRect = self.generateRect(newX, newY)

        # submit collision detection requests
        return [coll.CollisionInfo("enemy", self.newRect), 
                coll.CollisionRequest(self.collReq, self.newRect, ["floor", "wall"]),
                coll.CollisionRequest(self.collHitReq, self.newRect, ["catAttack"])]

    def updatePostColl(self, catInfo, collResponses):
        # process hit collision
        collHitResp = collResponses[self.collHitReq]
        for chr in collHitResp:
            if chr.intersects:
                if chr.collType == "catAttack":
                    if self.health > 0 and not self.beingHit:
                        self.health -= hitDamage
                        self.beingHit = True

                        if self.health <= 0:
                            bgChannels.scoreChannel.send((20, True))
            else:
                self.beingHit = False


        # process floor collision
        collResp = collResponses[self.collReq]
        for cr in collResp:
            if cr.intersects:
                if cr.collType == "floor":
                    self.move(self.newRect.Left, self.newRect.Top)
                    self.unstickCollision(self.newRect.Left, self.newRect.Top, self.newRect, cr.collRect)
                    self.velY = 0.0
                elif cr.collType == "wall":
                    self.velX *= -1
            else:   
                self.move(self.newRect.Left, self.newRect.Top)

    def unstickCollision(self, x, y, rect, collRect):
        (transX, transY) = coll.minTranslation(rect, collRect)
        if transY == 0.0:
            self.move(x - transX, self.y)
        else:
            self.move(self.x, y - transY)

def enemy1Tasklet(multipleWaves = True):
    enemies = []
    enemies += [Enemy1(start1X, start1Y), Enemy1(start2X, start2Y), Enemy1(start3X, start3Y)]

    initTime = time.time()
    spawnedWave2 = False
    spawnedWave3 = False

    while 1:
        catInfo = bgChannels.catInfoChannel.receive()

        # remove dead enemies
        enemies = [e for e in enemies if not e.isDead()]

        # spawn new enemies
        if multipleWaves:
            if not spawnedWave2 and time.time() - initTime > wave2Time:
                enemies += [Enemy1(start1X, start1Y), Enemy1(start2X, start2Y), Enemy1(start3X, start3Y)]
                spawnedWave2 = True
            if not spawnedWave3 and time.time() - initTime > wave3Time:
                enemies += [Enemy1(start1X, start1Y), Enemy1(start2X, start2Y), Enemy1(start3X, start3Y)]
                spawnedWave3 = True

        if enemies == []:
            continue

        # update enemies pre-collision
        collInfoReqs = []
        for e in enemies:
            collInfoReqs += e.updatePreColl(catInfo)

        # submit collision info/requests
        bgChannels.collisionsChannel.send(collInfoReqs)
        collResponses = bgChannels.collisionResponsesChannel.receive()

        # update enemies post-collision
        for e in enemies:
            e.updatePostColl(catInfo, collResponses)

        # draw enemies
        for e in enemies:
            e.draw()
