import stackless as ss
from PySFML import sf
from settings.settings import *
import globals.globals as g
import channels.channels as channels
from . import channels as bgChannels
import graphics.graphics as graphics
import graphics.animation as animation
from . import collisions as coll
from .types import *
import time
import math

flyingAnimDesc = (dataPath + "enemies/fish-plane/fish-plane", 2, 0.10)

gravity = 2000.0

startX   = 750.0
startY   = 250.0
startY2  = 350.0
startY3  = 450.0

wave2Time = 13.0
wave3Time = 26.0

moveVel = 60.0

rectWidth   = 73.0
rectHeight  = 53.0
rectOffsetX = 30.0
rectOffsetY = -50.0

projRectWidth   = 33.0
projRectHeight  = 33.0
projRectOffsetX = -17.0
projRectOffsetY = -17.0

initialHealth = 100
healthOffsetX = 10.0
healthOffsetY = -65.0
healthUnitWidth = 1.0
healthHeight = 2.0

hitDamage = 25

fireDelay = 2.0 # seconds

projSpeed = 500.0
projRotSpeed = 400.0

bubbleAttackImg = sf.Image()
bubbleAttackImg.LoadFromFile(dataPath + "enemies/fish-plane/bubble-attack.png")
bubbleAttackSpr = sf.Sprite(bubbleAttackImg)

bubbleAttackSpr.SetCenter(23.0, 19.0)

class Enemy2Proj:
    id = 0

    def __init__(self, x, y, catInfo):
        self.x = x
        self.y = y
        self.dir = math.atan2(catInfo.y - y, catInfo.x - x)
        self.rot = 0.0

        self.collReq = "enemy2Proj" + str(Enemy2Proj.id)
        Enemy2Proj.id += 1

    def updatePreColl(self, catInfo):
        timeNow = time.time()

        # new position
        self.x = self.x + projSpeed * math.cos(self.dir) * g.timeDelta
        self.y = self.y + projSpeed * math.sin(self.dir) * g.timeDelta
        self.newRect = self.generateRect(self.x, self.y)

        self.rot += projRotSpeed * g.timeDelta
        while self.rot > 360.0:
            self.rot -= 360.0

        # return collision detection requests
        return [coll.CollisionInfo("enemyAttack", self.newRect), 
                coll.CollisionRequest(self.collReq, self.newRect, ["cat"])]

    def updatePostColl(self, catInfo, collResponses):
        if self.collReq not in collResponses:
            return

        # process hit collision
        collHitResp = collResponses[self.collReq]
        for chr in collHitResp:
            if chr.intersects:
                if chr.collType == "cat":
                    #print("HIT")
                    pass

    def generateRect(self, x, y):
        return sf.FloatRect(x + projRectOffsetX, y + projRectOffsetX,
                            x + projRectOffsetX + projRectWidth,
                            y + projRectOffsetY + projRectHeight)

    def draw(self):
        #graphics.drawRect(g.window, self.generateRect(self.x, self.y), sf.Color.Green)
        bubbleAttackSpr.SetRotation(self.rot)
        bubbleAttackSpr.SetPosition(self.x, self.y)
        g.window.Draw(bubbleAttackSpr)

    def isDead(self):
        return (self.x < 0.0 or self.x > windowWidth or self.y < 0.0 or self.y > windowHeight)

class Enemy2:
    id = 0

    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.direction = Direction.LEFT
        self.rect = self.generateRect(x, y)
        self.velX = 0.0
        self.velY = 0.0
        self.health = initialHealth
        self.beingHit = False
        self.lastFireTime = time.time() - fireDelay

        self.flyingAnim = animation.Animation(flyingAnimDesc, True)
        self.anim = self.flyingAnim

        self.collReq = "enemy2Req" + str(Enemy2.id)
        self.collHitReq = "enemy2HitReq" + str(Enemy2.id)
        Enemy2.id += 1

    def isDead(self):
        return self.health <= 0

    def draw(self):
        if self.health > 0:
            healthRect = self.generateHealthRect()
            graphics.drawRect(g.window, healthRect, sf.Color.Red)
        #graphics.drawRect(g.window, self.rect, sf.Color.Blue)

        self.anim.setFlipped(self.direction == Direction.RIGHT)
        self.anim.draw(self.x, self.y)

    def generateRect(self, x, y):
        return sf.FloatRect(x + rectOffsetX, y + rectOffsetY,
                            x + rectOffsetX + rectWidth,
                            y + rectOffsetY + rectHeight)

    def generateHealthRect(self):
        return sf.FloatRect(self.x + healthOffsetX, self.y + healthOffsetY,
                            self.x + healthOffsetX + self.health * healthUnitWidth,
                            self.y + healthOffsetY + healthHeight)

    def move(self, x, y):
        self.x = x
        self.y = y
        self.rect = self.generateRect(x, y)

    def runAI(self, catInfo):
        timeNow = time.time()

        if self.direction == Direction.LEFT:
            self.velX = -moveVel
        else:
            self.velX = moveVel

    def updatePreColl(self, catInfo):
        timeNow = time.time()

        self.runAI(catInfo)

        # new position
        newX = self.x + self.velX * g.timeDelta
        newY = self.y + self.velY * g.timeDelta
        self.newRect = self.generateRect(newX, newY)

        # fireball
        if timeNow - self.lastFireTime > fireDelay:
            bgChannels.projAddChannel.send(Enemy2Proj(newX, newY, catInfo))
            self.lastFireTime = timeNow

        self.flyingAnim.update(g.timeDelta)

        # return collision detection requests
        return [coll.CollisionInfo("enemy", self.newRect), 
                coll.CollisionRequest(self.collReq, self.newRect, ["wall"]),
                coll.CollisionRequest(self.collHitReq, self.newRect, ["catAttack"])]

    def updatePostColl(self, catInfo, collResponses):
        # process hit collision
        collHitResp = collResponses[self.collHitReq]
        for chr in collHitResp:
            if chr.intersects:
                if chr.collType == "catAttack":
                    if self.health > 0 and not self.beingHit:
                        self.health -= hitDamage
                        self.beingHit = True

                        if self.health <= 0:
                            bgChannels.scoreChannel.send((20, True))
            else:
                self.beingHit = False

        # process floor collision
        collResp = collResponses[self.collReq]
        for cr in collResp:
            if cr.intersects:
                if cr.collType == "wall":
                    self.unstickCollision(self.newRect.Left - rectOffsetX, self.newRect.Top - rectOffsetY, self.newRect, cr.collRect)
                    self.direction = Direction.LEFT if self.direction == Direction.RIGHT else Direction.RIGHT
            else:   
                self.move(self.newRect.Left - rectOffsetX, self.newRect.Top - rectOffsetY)

    def unstickCollision(self, x, y, rect, collRect):
        (transX, transY) = coll.minTranslation(rect, collRect)
        if transY == 0.0:
            self.move(x - transX, self.y)
        else:
            self.move(self.x, y - transY)

def enemy2Tasklet(multipleWaves = True):
    enemies = []
    enemies.append(Enemy2(startX, startY))

    projectiles = []

    initTime = time.time()
    spawnedWave2 = False
    spawnedWave3 = False

    while 1:
        catInfo = bgChannels.catInfoChannel.receive()

        # remove dead enemies
        enemies = [e for e in enemies if not e.isDead()]

        # spawn new enemies
        if multipleWaves:
            if not spawnedWave2 and time.time() - initTime > wave2Time:
                enemies.append(Enemy2(startX, startY2))
                spawnedWave2 = True
            if not spawnedWave3 and time.time() - initTime > wave3Time:
                enemies.append(Enemy2(startX, startY3))
                spawnedWave3 = True

        if enemies == []:
            continue

        # update enemies pre-collision
        collInfoReqs = []
        for e in enemies:
            collInfoReqs += e.updatePreColl(catInfo)

        # submit collision info/requests
        bgChannels.collisionsChannel.send(collInfoReqs)
        collResponses = bgChannels.collisionResponsesChannel.receive()

        # update enemies post-collision
        for e in enemies:
            e.updatePostColl(catInfo, collResponses)

        # draw enemies
        for e in enemies:
            e.draw()
