import stackless as ss
from PySFML import sf
from settings.settings import *
import globals.globals as g
import channels.channels as channels
from . import channels as bgChannels
import graphics.graphics as graphics
from . import collisions as coll
from .types import *
import time
import math

class ProjType:
    ARC = 0
    STRAIGHT = 1

gravity = 2000.0

start1X   = 50.0
start1Y   = 450.0
start2X   = 920.0
start2Y   = 450.0

walkVel = 20.0

rectWidth   = 53.0
rectHeight  = 87.0
rectOffsetX = 0.0
rectOffsetY = 0.0

projRectWidth   = 33.0
projRectHeight  = 33.0
projRectOffsetX = -17.0
projRectOffsetY = -17.0

healthOffsetX = -10.0
healthOffsetY = -50.0
healthUnitWidth = 1.0
healthHeight = 2.0

attackOffsetX = 0.0
attackOffsetY = rectHeight
attackWidth = 500.0
attackHeight = 50.0

initialHealth = 100
hitDamage = 6
chargeUpDelay = 1.0     # seconds
attackDist = 80.0
attackDuration = 0.2    # seconds
attackDelay = 2.0   # seconds

projSlowFactor = 2.0
straightProjVelX = 1000.0
projRotSpeed = 400.0

turretImg = sf.Image()
turretImg.LoadFromFile(dataPath + "enemies/fish-turret/fish-turret.png")
turretSpr = sf.Sprite(turretImg)

bubbleAttackImg = sf.Image()
bubbleAttackImg.LoadFromFile(dataPath + "enemies/fish-plane/bubble-attack.png")
bubbleAttackSpr = sf.Sprite(bubbleAttackImg)
bubbleAttackSpr.SetCenter(23.0, 19.0)

class Enemy3Proj:
    id = 0

    def __init__(self, x, y, catInfo, type):
        self.x = x
        self.y = y
        self.dir = math.atan2(catInfo.y - y, catInfo.x - x)
        self.type = type
        self.rot = 0.0

        dist = catInfo.x - x
        distSign = 1.0 if dist >= 0.0 else -1.0

        if type == ProjType.ARC:
            vel = math.fabs(dist) * gravity / 2.0
            self.velX = distSign * math.sqrt(vel)
            self.velY = -math.fabs(math.sqrt(vel))
        else:
            self.velX = distSign * straightProjVelX
            self.velY = 0.0

        self.collReq = "enemy3Proj" + str(Enemy3Proj.id)
        Enemy3Proj.id += 1

    def updatePreColl(self, catInfo):
        timeNow = time.time()

        # new position
        self.x = self.x + self.velX * (g.timeDelta / projSlowFactor)
        self.y = self.y + self.velY * (g.timeDelta / projSlowFactor)

        if self.type == ProjType.ARC:
            self.velY += gravity * (g.timeDelta / projSlowFactor)

        self.newRect = self.generateRect(self.x, self.y)

        self.rot += projRotSpeed * g.timeDelta
        while self.rot > 360.0:
            self.rot -= 360.0

        # return collision detection requests
        return [coll.CollisionInfo("enemyAttack", self.newRect), 
                coll.CollisionRequest(self.collReq, self.newRect, ["cat"])]

    def updatePostColl(self, catInfo, collResponses):
        if self.collReq not in collResponses:
            return

        # process hit collision
        collHitResp = collResponses[self.collReq]
        for chr in collHitResp:
            if chr.intersects:
                if chr.collType == "cat":
                    #print("HIT")
                    pass

    def generateRect(self, x, y):
        return sf.FloatRect(x + projRectOffsetX, y + projRectOffsetY,
                            x + projRectOffsetX + projRectWidth,
                            y + projRectOffsetY + projRectHeight)

    def draw(self):
        #graphics.drawRect(g.window, self.generateRect(self.x, self.y), sf.Color.Green)
        bubbleAttackSpr.SetRotation(self.rot)
        bubbleAttackSpr.SetPosition(self.x, self.y)
        g.window.Draw(bubbleAttackSpr)

    def isDead(self):
        return (self.x < 0.0 or self.x > windowWidth or self.y < 0.0 or self.y > windowHeight)

class Enemy3:
    id = 0

    def __init__(self, x, y, flip=False):
        self.x = x
        self.y = y
        self.direction = Direction.RIGHT
        self.rect = self.generateRect(x, y)
        self.velX = 0.0
        self.velY = 0.0
        self.health = initialHealth
        self.beingHit = False
        self.flip = flip

        self.attackRect = None
        self.lastChargeUpTime = time.time()
        self.lastAttackTime = time.time()
        self.chargingUp = False
        self.attacking = False
        self.prevShotType = ProjType.ARC

        self.collReq = "enemy3Req" + str(Enemy3.id)
        self.collHitReq = "enemy3HitReq" + str(Enemy3.id)
        Enemy3.id += 1

    def isDead(self):
        return self.health <= 0

    def draw(self):
        if self.health > 0:
            healthRect = self.generateHealthRect()
            graphics.drawRect(g.window, healthRect, sf.Color.Red)
        #graphics.drawRect(g.window, self.rect, sf.Color.Blue)
        
        turretSpr.FlipX(self.flip)
        if self.flip:
            turretSpr.SetPosition(self.x - 45.0, self.y - 40.0)
        else:
            turretSpr.SetPosition(self.x - 20.0, self.y - 40.0)
        g.window.Draw(turretSpr)

        if self.attackRect is not None:
            graphics.drawRect(g.window, self.attackRect, sf.Color.Green)

    def generateRect(self, x, y):
        return sf.FloatRect(x + rectOffsetX, y + rectOffsetY,
                            x + rectOffsetX + rectWidth,
                            y + rectOffsetY + rectHeight)

    def generateHealthRect(self):
        return sf.FloatRect(self.x + healthOffsetX, self.y + healthOffsetY,
                            self.x + healthOffsetX + self.health * healthUnitWidth,
                            self.y + healthOffsetY + healthHeight)

    def generateAttackRect(self):
        if self.direction == Direction.LEFT:
            return sf.FloatRect(self.x - attackOffsetX - attackWidth,
                                self.y + attackOffsetY - attackHeight,
                                self.x - attackOffsetX,
                                self.y + attackOffsetY)
        else:
            return sf.FloatRect(self.x + rectWidth + attackOffsetX,
                                self.y + attackOffsetY - attackHeight,
                                self.x + rectWidth + attackOffsetX + attackWidth,
                                self.y + attackOffsetY)

    def move(self, x, y):
        self.x = x
        self.y = y
        self.rect = self.generateRect(x, y)

    def runAI(self, catInfo):
        timeNow = time.time()

        # charge up attack
        if catInfo.onGround:
            if not self.chargingUp and timeNow - self.lastChargeUpTime >= attackDelay:
                self.lastChargeUpTime = time.time()
                self.chargingUp = True
                self.velX = 0.0

        if self.chargingUp and timeNow - self.lastChargeUpTime >= chargeUpDelay and not self.attacking:
            self.lastAttackTime = timeNow
            self.chargingUp = False
            self.attacking = True

        if self.attacking:
            if self.prevShotType == ProjType.ARC:
                bgChannels.projAddChannel.send(Enemy3Proj(self.x, self.y, catInfo, ProjType.STRAIGHT))
                self.prevShotType = ProjType.STRAIGHT
            else:
                bgChannels.projAddChannel.send(Enemy3Proj(self.x, self.y, catInfo, ProjType.ARC))
                self.prevShotType = ProjType.ARC

            self.attacking = False

    def updatePreColl(self, catInfo):
        timeNow = time.time()

        self.velY += gravity * g.timeDelta
        self.runAI(catInfo)

        # new position
        newX = self.x + self.velX * g.timeDelta
        newY = self.y + self.velY * g.timeDelta

        # prevent enemy from falling through floor
        if newY > windowHeight:
            newY = startY
            self.velY = 0.0

        self.newRect = self.generateRect(newX, newY)

        # submit collision detection requests
        collReq =  [coll.CollisionInfo("enemy", self.newRect), 
                    coll.CollisionRequest(self.collReq, self.newRect, ["floor", "wall"]),
                    coll.CollisionRequest(self.collHitReq, self.newRect, ["catAttack"])]

        if self.attackRect is not None:
            collReq.append(coll.CollisionInfo("enemyAttack", self.attackRect))

        return collReq

    def updatePostColl(self, catInfo, collResponses):
        # process hit collision
        collHitResp = collResponses[self.collHitReq]
        for chr in collHitResp:
            if chr.intersects:
                if chr.collType == "catAttack":
                    if self.health > 0 and not self.beingHit:
                        self.health -= hitDamage
                        self.beingHit = True

                        if self.health <= 0:
                            bgChannels.scoreChannel.send((20, True))
            else:
                self.beingHit = False


        # process floor collision
        collResp = collResponses[self.collReq]
        for cr in collResp:
            if cr.intersects:
                if cr.collType == "floor":
                    self.move(self.newRect.Left, self.newRect.Top)
                    self.unstickCollision(self.newRect.Left, self.newRect.Top, self.newRect, cr.collRect)
                    self.velY = 0.0
                elif cr.collType == "wall":
                    self.velX *= -1
            else:   
                self.move(self.newRect.Left, self.newRect.Top)

    def unstickCollision(self, x, y, rect, collRect):
        (transX, transY) = coll.minTranslation(rect, collRect)
        if transY == 0.0:
            self.move(x - transX, self.y)
        else:
            self.move(self.x, y - transY)

def enemy3Tasklet(multipleWaves = True):
    enemies = []
    enemies += [Enemy3(start1X, start1Y), Enemy3(start2X, start2Y, True)]

    while 1:
        catInfo = bgChannels.catInfoChannel.receive()

        # remove dead enemies
        enemies = [e for e in enemies if not e.isDead()]

        if enemies == []:
            continue

        # update enemies pre-collision
        collInfoReqs = []
        for e in enemies:
            collInfoReqs += e.updatePreColl(catInfo)

        # submit collision info/requests
        bgChannels.collisionsChannel.send(collInfoReqs)
        collResponses = bgChannels.collisionResponsesChannel.receive()

        # update enemies post-collision
        for e in enemies:
            e.updatePostColl(catInfo, collResponses)

        # draw enemies
        for e in enemies:
            e.draw()
