import stackless as ss
from PySFML import sf
from settings.settings import *
import globals.globals as g
import graphics.graphics as graphics
import graphics.animation as animation
import channels.channels as channels
from . import channels as bgChannels
from . import collisions as coll
from .types import *
import random

walkSpeed = 150

rectOffsetX = 3.0
rectOffsetY = 2.0

timeOffsetY = -2.0
timeHeight = 2.0
groundTime = 10.0

zombieWalkAnimDesc = (dataPath + "cannibalArt/ZMwalk", 6, 0.2)
fireballAnimDesc = (dataPath + "cannibalArt/fireball", 1, 0.3)

class StickType:
    NONE    = 0
    STAY    = 1
    WALK    = 2

class Food:
    def __init__(self, fallSpeed, spr, x, number, value, stick, foodType):
        self.x = x
        self.y = 0
        self.fallSpeed = fallSpeed
        self.value = value
        self.valid = True
        self.eaten = False
        self.stick = stick
        self.onGround = False
        self.groundTime = 0.0
        self.foodSpr = spr
        self.rect = self.generateRect(x,0)
        self.walk = 1
        self.type = foodType
        self.name = "food" + str(number)
        if self.stick == StickType.WALK:
            self.anim = animation.Animation(zombieWalkAnimDesc, True)
        if self.stick == StickType.STAY:
            self.anim = animation.Animation(fireballAnimDesc, True)

    def draw(self):
        #graphics.drawRect(g.window, self.generateRect(self.x, self.y), sf.Color.Cyan)
        if (self.stick == StickType.STAY):
            (w, h) = self.foodSpr.GetSize()
            self.anim.draw(self.x + w / 2 - rectOffsetX, self.y + h / 2 - rectOffsetY, True)
            if self.onGround:
                graphics.drawRect(g.window, self.generateTimeRect(self.x, self.y), sf.Color.Red)
        elif (self.onGround):
            if self.stick == StickType.WALK:
                (w, h) = self.foodSpr.GetSize()
                self.anim.setFlipped(self.walk == -1)
                self.anim.draw(self.x + w / 2 - rectOffsetX, self.y + h / 2 - rectOffsetY, True)
            graphics.drawRect(g.window, self.generateTimeRect(self.x, self.y), sf.Color.Red)
        else:
            self.foodSpr.SetPosition(self.x, self.y)
            self.foodSpr.FlipX(self.walk == -1)
            g.window.Draw(self.foodSpr)
                

    def generateTimeRect(self, x, y):
        (w, h) = self.foodSpr.GetSize()
        return sf.FloatRect(x, y + timeOffsetY,
                            x + (groundTime - self.groundTime) * (w / groundTime),
                            y + timeOffsetY + timeHeight)

    def generateRect(self, x, y):
        (w, h) = self.foodSpr.GetSize()
        return sf.FloatRect(x + rectOffsetX, y + rectOffsetY,
                            x + w - rectOffsetX,
                            y + h - rectOffsetY)

    def update(self, time):
        if (self.stick == StickType.STAY):
            self.anim.update(g.timeDelta)
        if (not(self.onGround)):
            self.y += self.fallSpeed * time
        else:
            self.groundTime += time
            if (self.stick == StickType.WALK):
                self.x += walkSpeed * time * self.walk
                self.anim.update(g.timeDelta)
            if (self.groundTime >= groundTime):
                self.valid = False
        self.rect = self.generateRect(self.x, self.y)

    def checkCollisions(self, collResponses):
        collResp = collResponses[self.name + "Crash"]
        for cr in collResp:
            if cr.intersects:
                if cr.collType == "floor":
                    if (self.stick == StickType.NONE):
                        self.valid = False
                        return
                    else:
                        self.onGround = True
                if cr.collType == "wall":
                    if (self.stick == StickType.WALK):
                        self.walk = -self.walk

        collResp = collResponses[self.name + "Eaten"]
        for cr in collResp:
            if cr.intersects:
                if cr.collType == "cat":
                    self.eaten = True
                    self.valid = False
                    return

def foodManagerTasklet():
    level = 0
    goals = [ 110, 110, 130, 200 ]
    dropSpeedMin = [ 0.5, 0.75, 0.75, 0.2 ]
    
    foodList = []
    foodValues = []
    foodStick = []
    foodTypes = []
    foodProbabilities = []
    foodSprites = []
    foodSpeeds = []

    # Load Good Food
    goodFood = sf.Image()
    goodFood.LoadFromFile(dataPath + "cannibalArt/mochicat0.png")
    goodFoodSpr = sf.Sprite(goodFood)
    foodSprites.append(goodFoodSpr)
    foodValues.append(1)
    foodStick.append(StickType.NONE)
    foodTypes.append(FoodType.POINT)
    foodSpeeds.append(150)

    medFood = sf.Image()
    medFood.LoadFromFile(dataPath + "cannibalArt/mediumcat0.png")
    medFoodSpr = sf.Sprite(medFood)
    foodSprites.append(medFoodSpr)
    foodValues.append(5)
    foodStick.append(StickType.NONE)
    foodTypes.append(FoodType.POINT)
    foodSpeeds.append(200)

    smallFood = sf.Image()
    smallFood.LoadFromFile(dataPath + "cannibalArt/smallcat0.png")
    smallFoodSpr = sf.Sprite(smallFood)
    foodSprites.append(smallFoodSpr)
    foodValues.append(10)
    foodStick.append(StickType.NONE)
    foodTypes.append(FoodType.POINT)
    foodSpeeds.append(250)

    # Load Bad Food
    badFood = sf.Image()
    badFood.LoadFromFile(dataPath + "cannibalArt/chopstix0.png")
    badFoodSpr = sf.Sprite(badFood)
    foodSprites.append(badFoodSpr)
    foodValues.append(-1)
    foodStick.append(StickType.NONE)
    foodTypes.append(FoodType.POINT)
    foodSpeeds.append(300)
    

    zombieFood = sf.Image()
    zombieFood.LoadFromFile(dataPath + "cannibalArt/zombieMochi0.png")
    zombieFoodSpr = sf.Sprite(zombieFood)
    foodSprites.append(zombieFoodSpr)
    foodValues.append(-10)
    foodStick.append(StickType.WALK)
    foodTypes.append(FoodType.POINT)
    foodSpeeds.append(150)

    fireball = sf.Image()
    fireball.LoadFromFile(dataPath + "cannibalArt/fireball0.png")
    fireballSpr = sf.Sprite(fireball)
    foodSprites.append(fireballSpr)
    foodValues.append(-5)
    foodStick.append(StickType.STAY)
    foodTypes.append(FoodType.POINT)
    foodSpeeds.append(100)

    # Load Power Ups
    speedUp = sf.Image()
    speedUp.LoadFromFile(dataPath + "cannibalArt/speedup0.png")
    speedUpSpr = sf.Sprite(speedUp)
    foodSprites.append(speedUpSpr)
    foodValues.append(0)
    foodStick.append(StickType.NONE)
    foodTypes.append(FoodType.SPEEDUP)
    foodSpeeds.append(175)

    jumpUp = sf.Image()
    jumpUp.LoadFromFile(dataPath + "cannibalArt/jumpup0.png")
    jumpUpSpr = sf.Sprite(jumpUp)
    foodSprites.append(jumpUpSpr)
    foodValues.append(0)
    foodStick.append(StickType.NONE)
    foodTypes.append(FoodType.JUMPUP)
    foodSpeeds.append(175)
    
    number = 0
    spawnTime = dropSpeedMin[level]
    nextSpawn = 0

    channels.broadcast(bgChannels.amountEatenChannel, goals[level])

    while 1:
        catInfo = bgChannels.catInfoChannel.receive()

        (layer, prog) = bgChannels.progressionChannel.receive()

        if prog:
            level = layer
            channels.broadcast(bgChannels.catResetChannel, goals[level])

        if level == 0:
            foodProbabilities = [ 60, 30, 0, 10, 0, 0, 0, 0 ]
        elif level == 1:
            foodProbabilities = [ 40, 20, 4.5, 30, 5.5, 0, 0, 0 ]
        elif level == 2:
            foodProbabilities = [ 0, 0, 44, 0, 20, 20, 8, 8 ]
        elif level == 3:
            foodProbabilities = [ 40, 25, 15, 0, 0, 0, 10, 10 ]
        
        time = g.timeDelta
        for f in foodList:
            f.update(time)

        collisionInfoReq = []
        for f in foodList:
            collisionInfoReq.append(coll.CollisionInfo(f.name, f.rect))
            collisionInfoReq.append(coll.CollisionRequest(f.name + "Crash", f.rect, ["floor", "wall"]))
            collisionInfoReq.append(coll.CollisionRequest(f.name + "Eaten", f.rect, ["cat"]))

        collResponses = dict()

        if (len(collisionInfoReq) != 0):
            bgChannels.collisionsChannel.send(collisionInfoReq)
            collResponses = bgChannels.collisionResponsesChannel.receive()
        
        for f in foodList:
            f.checkCollisions(collResponses)

        eatenList = [f for f in foodList if f.eaten]
        channels.broadcast(bgChannels.amountEatenChannel, eatenList)
        
        foodList = [f for f in foodList if f.valid]

        nextSpawn -= time
        if (nextSpawn <= 0):
            number += 1
            rand = random.randrange(0, 100)
            index = 0
            while (index >= 0):
                if (rand < foodProbabilities[index]):
                    speedRand = random.randrange(-2, 2)
                    foodList.append(Food(foodSpeeds[index] + 10 * speedRand, foodSprites[index], random.randrange(10, windowWidth - 80), number, foodValues[index], foodStick[index], foodTypes[index]))
                    break
                else:
                    rand -= foodProbabilities[index]
                    index += 1
                    
            nextSpawn = dropSpeedMin[level]

        third = goals[level] / 3
        sun = 0
        if (catInfo.value < third):
            sun = 0
        elif catInfo.value < 2 * third:
            sun = 1
        else:
            sun = 2

        channels.broadcast(bgChannels.backgroundChannel, (level+1, sun))
        
        for f in foodList:
            f.draw()
