from PySFML import sf
import globals.globals as globals

fileExtension = ".png"

loadedSprites = dict()

def loadSprite(filePath):
    if filePath not in loadedSprites:
        image = sf.Image()
        image.LoadFromFile(filePath)
        loadedSprites[filePath] = sf.Sprite(image)

    return loadedSprites[filePath]


class Animation:
    def __init__(self, animDescription, loop=True):
        self.loop = loop
        self.finished = False
        self.flipped = False

        (baseFilePath, lastFrameIndex, frameDuration) = animDescription

        self.sprites = []
        for i in range(lastFrameIndex + 1):
            sprite = loadSprite(baseFilePath + str(i) + fileExtension)
            self.sprites.append(sprite)

        self.width = self.sprites[0].GetImage().GetWidth()
        self.height = self.sprites[0].GetImage().GetHeight()
        self.scale = 1.0

        self.lastFrameIndex = lastFrameIndex
        self.currentFrame = 0
        self.frameDuration = frameDuration
        self.timeElapsed = 0.0

    def update(self, timeDelta):
        if self.finished:
            return

        self.timeElapsed += timeDelta

        if self.timeElapsed >= self.frameDuration:
            self.currentFrame += 1
            if self.currentFrame > self.lastFrameIndex:
                if self.loop:
                    self.currentFrame = 0
                else:
                    self.currentFrame -= 1
                    self.finished = True
            self.sprites[self.currentFrame].SetScale(self.scale, self.scale)
            self.timeElapsed -= self.frameDuration

    def draw(self, x, y, center=False, rot=0.0):
        self.sprites[self.currentFrame].SetCenter(self.width / 2, self.height / 2.0)
        self.sprites[self.currentFrame].SetRotation(rot)

        if center:
            self.sprites[self.currentFrame].SetPosition(x, y)
        else:
            self.sprites[self.currentFrame].SetPosition(x + self.width / 2, y)
        globals.window.Draw(self.sprites[self.currentFrame])

    def reset(self):
        self.currentFrame = 0
        self.timeElapsed = 0.0
        self.finished = False

    def setScale(self, scale):
        self.scale = scale
        self.sprites[self.currentFrame].SetScale(scale, scale)

    def resize(self, width, height):
        self.sprites[self.currentFrame].Resize(width, height)

    def setFlipped(self, flipped):
        for spr in self.sprites:
            spr.FlipX(flipped)
