import stackless as ss
from PySFML import sf
from settings.settings import *
import globals.globals as g
import channels.channels as channels
from . import channels as bgChannels
import graphics.graphics as graphics
import graphics.animation as animation
from . import collisions as coll
from .types import *
import time
import random

topThickness = 5
thickness = 10
level = 0

asteroidDesc = (dataPath + "asteroids/asteroid", 0, 0.1)
flamingAsteroidDesc = (dataPath + "asteroids/fasteroid", 2, 0.2)

class Block:
    def __init__(self, x, y, w, h, velX, isDamaging):
        self.x = x
        self.y = y
        self.w = w
        self.h = h
        self.velX = velX
        self.isDamaging = isDamaging
        self.generateRects()
        if self.isDamaging:
            self.anim = animation.Animation(flamingAsteroidDesc, True)
        else:
            self.anim = animation.Animation(asteroidDesc, False)
    def generateRects(self):
        self.rect = sf.FloatRect(self.x, self.y, self.x + self.w, self.y + self.h)
    def draw(self):
        windowSize = windowWidth * (1 - ((level) * 0.25))
        alpha = (self.x - (windowWidth - windowSize))/ windowSize
        if alpha < 0:
            alpha = 0
        self.anim.sprites[self.anim.currentFrame].SetColor(sf.Color(255, 255, 255, int(255 * alpha)))
        if self.isDamaging:
            self.anim.resize(int(self.w * 1.4), int(self.h * 1.4))
        else:
            self.anim.resize(self.w, self.h)
        self.anim.draw(self.x, self.y + self.h / 2)
        
    def update(self, collisionList):
        self.move(self.x + self.velX * g.timeDelta, self.y)
        collisionList.append(coll.CollisionInfo("block", self.rect, (self.velX, self.isDamaging)))
        self.anim.update(g.timeDelta)
    
    def isOffScreen(self):
        return self.x + self.w < 0

    def move(self, x, y):
        self.x = x
        self.y = y
        self.generateRects()

def blockTasklet():
    global level
    blocks = []
    goals = [100, 100, 100, 100]
    random.seed(time.time())
    while 1:
        collisionList = []
        (level, prog) = bgChannels.levelChannel.receive()
        if prog:
            bgChannels.goalChannel.send(goals[level]);
        blocks = [b for b in blocks if not b.isOffScreen()]
        if random.randint(1,100) < 5 and len(blocks) < 20:
            n = random.randint(1,4)
            for x in range(n):
                rng = (random.randint(1, 100) + random.randint(1, 100)) / 2
                if rng >= 30 and rng <= 70:
                    y = random.randint(400, 600)
                else:
                    y = random.randint(0, 399)
                if random.randint(1, 100) <= 10 * (level + 1):
                    blocks.append(Block(windowWidth + random.randint(0, 100), y, random.randint(90, 100), random.randint(75, 100), -random.randint(100,200), True))
                else:
                    blocks.append(Block(windowWidth + random.randint(0, 100), y, random.randint(90, 100), random.randint(75, 100), -random.randint(100,200), False))
        for b in blocks:
            b.draw()
        for b in blocks:
            b.update(collisionList)
        bgChannels.collisionsChannel.send(collisionList)
        
