import stackless as ss
from PySFML import sf
from settings.settings import *
import globals.globals as g
import channels.channels as channels
from . import channels as bgChannels
import graphics.graphics as graphics
import time

class Progress:
    FORWARDS    = 0
    BACKWARDS   = 1
    STAY        = 2

layerStr = sf.String("Layer: ")
layerStr.SetColor(sf.Color.White)
layerStr.SetPosition(20.0, 60.0)
layerStr.SetSize(15.0)

statusStr = sf.String("Status: ")
statusStr.SetColor(sf.Color.White)
statusStr.SetPosition(20.0, 90.0)
statusStr.SetSize(15.0)

progressStr = sf.String("||")
progressStr.SetColor(sf.Color.White)
progressStr.SetPosition(500.0, 40.0)
progressStr.SetSize(15.0)

progressUpImg = sf.Image()
progressUpImg.LoadFromFile(dataPath + "progress/ProgressUp.png")
progressUpSpr = sf.Sprite(progressUpImg)

progressNoImg = sf.Image()
progressNoImg.LoadFromFile(dataPath + "progress/ProgressNo.png")
progressNoSpr = sf.Sprite(progressNoImg)

progressDownImg = sf.Image()
progressDownImg.LoadFromFile(dataPath + "progress/ProgressDown.png")
progressDownSpr = sf.Sprite(progressDownImg)

noticeSpr = progressUpSpr

sadCatImg = sf.Image()
sadCatImg.LoadFromFile(dataPath + "progress/pi-sad.png")
sadCatSpr = sf.Sprite(sadCatImg)
sadCatSpr.SetPosition(500.0, 40.0)

indCatImg = sf.Image()
indCatImg.LoadFromFile(dataPath + "progress/pi-ind.png")
indCatSpr = sf.Sprite(indCatImg)
indCatSpr.SetPosition(500.0, 40.0)

hapCatImg = sf.Image()
hapCatImg.LoadFromFile(dataPath + "progress/pi-happy.png")
hapCatSpr = sf.Sprite(hapCatImg)
hapCatSpr.SetPosition(500.0, 40.0)

catSpr = [sadCatSpr, indCatSpr, hapCatSpr]

noticeDuration = 1.5    # secs

music = []
progress = Progress.FORWARDS
musicIndex = 0
layers = 4
volume = 100.0
timePlayed = 0.0
noticeTimeStamp = time.time()

def drawStatus(progression):
    global music
    global progress
    global musicIndex
    global timePlayed

    def progressToStr(progress):
        if progress == Progress.FORWARDS:
            return ">>"
        elif progress == Progress.BACKWARDS:
            return "<<"
        else:
            return "||"

    def timeFormat(time):
        return "%.1f" % time


    layerStr.SetText("Progression: " + str(musicIndex + 1))
    layerStr.SetPosition(100.0, 15.0)
    g.window.Draw(layerStr)

    playingOffset = timePlayed
    duration = music[musicIndex].GetDuration()

    statusStr.SetPosition(850.0, 15.0)
    statusStr.SetText(timeFormat(playingOffset) + "/" +
                      timeFormat(duration))
    g.window.Draw(statusStr)

    x = 199.0
    y = 18.0
    tickLeft = sf.FloatRect(x, y, x + 1.0, y + 14.0)
    graphics.drawRect(g.window, tickLeft, sf.Color(100, 100, 100))

    x = 824.0
    y = 18.0
    tickRight = sf.FloatRect(x, y, x + 1.0, y + 14.0)
    graphics.drawRect(g.window, tickRight, sf.Color(100, 100, 100))

    x = 200.0
    y = 20.0
    maxWidth = 1024.0 - 2 * x

    width = (maxWidth / duration) * playingOffset

    playbackRect = sf.FloatRect(x, y, x + width, y + 10)
    graphics.drawRect(g.window, playbackRect, sf.Color.White)

    #progressStr.SetText(progressToStr(progress))
    g.window.Draw(catSpr[progression])

    if time.time() - noticeTimeStamp < noticeDuration:
        noticeSpr.SetPosition(400.0, 300.0)
        g.window.Draw(noticeSpr)

def loadMusic():
    music = []
    for i in range(layers):
        music.append(sf.Music())
        music[i].OpenFromFile(dataPath + "music/blocks/BlocksL" + str(i + 1) + ".ogg")
        music[i].Initialize(2, 44100)
        music[i].SetLoop(True)
        music[i].SetVolume(volume)

    return music

def musicTasklet():
    global music
    global progress
    global musicIndex
    global timePlayed

    try:
        music = loadMusic()
        musicIndex = 0
        music[musicIndex].Play()

        layerStr.SetText("Progression: " + str(musicIndex + 1))
        while 1:
            inputState = channels.inputChannel.receive()
        #if inputState.keyPressed("f1"):
        #    progress = Progress.BACKWARDS
        #elif inputState.keyPressed("f2"):
        #    progress = Progress.STAY
        #elif inputState.keyPressed("f3"):
        #    progress = Progress.FORWARDS
            prog = False

            if inputState.keyPressed("f4"):
                progress = Progress.FORWARDS
                musicProgress()
                prog = True

            progression = bgChannels.progressionChannel.receive()
            if progression == 0:
                progress = Progress.BACKWARDS
            elif progression == 1:
                progress = Progress.STAY
            else:
                progress = Progress.FORWARDS

            timePlayed += g.timeDelta


            if music[musicIndex].GetDuration() - timePlayed <= 0.1:
                musicProgress()
                prog = True

            drawStatus(progression)
            channels.broadcast(bgChannels.levelChannel, (musicIndex, prog))
    except TaskletExit:
        music[musicIndex].Stop()

def musicProgress():
    global music
    global progress
    global musicIndex
    global timePlayed
    global noticeSpr
    global noticeTimeStamp

    if progress == Progress.FORWARDS:
        print("\tforwards!")
        newMusicIndex = musicIndex + 1
    elif progress == Progress.BACKWARDS:
        print("\tbackwards!")
        newMusicIndex = musicIndex - 1
    elif progress == Progress.STAY:
        #print("\tstay!")
        newMusicIndex = musicIndex

    if newMusicIndex < 0:
        newMusicIndex = 0
    elif newMusicIndex >= len(music):
        newMusicIndex = len(music) - 1
        g.gameDone = True
        g.gameWon = True

    music[musicIndex].Stop()
    music[newMusicIndex].Play()

    musicIndex = newMusicIndex

    timePlayed = 0.0

    if progress == Progress.FORWARDS:
        noticeSpr = progressUpSpr
    elif progress == Progress.BACKWARDS:
        noticeSpr = progressDownSpr
    elif progress == Progress.STAY:
        noticeSpr= progressNoSpr

    noticeTimeStamp = time.time()
