import stackless as ss
from PySFML import sf
import sys
import channels.channels as channels
from . import channels as bgChannels
import graphics.graphics as graphics
import graphics.animation as animation
import platformer.platformerSettings as pS
from settings.settings import *
import globals.globals as g
from . import collisions as coll
from .types import *
import time

rectWidth   = 53.0
rectHeight  = 37.0
rectOffsetX = 5.0
rectOffsetY = -5.0

idleAnimDesc = (dataPath + "mochicat/idle/mochiIdle", 0, 1.0)
walkAnimDesc = (dataPath + "mochicat/walk/mochiWalk", 3, 0.1)
jumpAnimDesc = (dataPath + "mochicat/jump/mochiJump", 1, 0.2)
fallAnimDesc = (dataPath + "mochicat/fall/mochiFall", 1, 0.2)
hurtAnimDesc = (dataPath + "mochicat/hurt/mochiHurt", 0, 0.2)

class Cat:
    def __init__(self, x, y):
        self.x = x
        self.y = y
        self.velY = 0.0
        self.velX = 0.0
        self.rect = self.generateRect(x, y)
        self.state = pS.State.ON_GROUND
        self.direction = Direction.RIGHT
        self.can_jump = False
        self.jumped = False
        self.is_invincible = False
        self.invincibleTimeLeft = 0
        self.draw_delta = 0
        self.should_draw = True
        self.goal = 0.0
        self.score = 0.0
        image = sf.Image()
        image.LoadFromFile(dataPath + "cats/mochicat0.png")
        self.sprite = sf.Sprite(image)

        # animations
        self.idleAnim = animation.Animation(idleAnimDesc, True)
        self.walkAnim = animation.Animation(walkAnimDesc, True)
        self.jumpAnim = animation.Animation(jumpAnimDesc, False)
        self.fallAnim = animation.Animation(fallAnimDesc, False)
        self.hurtAnim = animation.Animation(hurtAnimDesc, False)
        self.anim = self.idleAnim

    def move(self, x, y):
        self.x = x
        self.y = y
        self.rect = self.generateRect(x, y)

    def draw(self):
        self.anim.update(g.timeDelta)
        self.anim.setFlipped(self.direction == Direction.RIGHT)
        self.draw_delta = self.draw_delta + g.timeDelta
        if self.draw_delta >= pS.draw_delta:
            self.should_draw = not self.should_draw
        if not self.is_invincible or self.should_draw:
          self.anim.draw(self.x, self.y)

    def generateRect(self, x, y):
        return sf.FloatRect(x, y,
                            x + rectOffsetX + rectWidth,
                            y + rectOffsetY + rectHeight)

    def reset(self, goal):
        self.goal = goal
        self.score = 0

    def update(self, inputState):
        timeNow = time.time()

        if self.is_invincible:
                self.invincibleTimeLeft = self.invincibleTimeLeft - g.timeDelta
                if self.invincibleTimeLeft <= 0:
                    self.is_invincible = False

        self.velX = 0.0
        if inputState.keyDown("left") or inputState.joyLeft:
            self.velX = -pS.velX
            self.direction = Direction.LEFT
        elif inputState.keyDown("right") or inputState.joyRight:
            self.velX = pS.velX
            self.direction = Direction.RIGHT
            
        if inputState.keyDown("c") or inputState.joyDown("jump"):
            if self.state == pS.State.ON_GROUND or self.state == pS.State.ON_BLOCK:
                self.velY = pS.velY
            elif self.state == pS.State.IN_AIR and self.can_jump:
                self.velY = pS.velY
                self.jumped = True
                self.can_jump = False
        else:
            if self.state == pS.State.IN_AIR and not self.jumped:
                self.can_jump = True
            

        self.velY += pS.gravity * g.timeDelta
        if self.velY >= pS.terminalVelocity:
            self.velY = pS.terminalVelocity

        newX = self.x + self.velX * g.timeDelta
        newY = self.y + self.velY * g.timeDelta
        newRect = self.generateRect(newX, newY)

        # submit collision detection requests
        collisionInfoReq = [coll.CollisionInfo("cat", newRect, None),
                            coll.CollisionRequest("catReq", newRect, ["floor", "leftwall", "rightwall", "block"])]
        bgChannels.collisionsChannel.send(collisionInfoReq)
        collResponses = bgChannels.collisionResponsesChannel.receive()

        self.state = pS.State.IN_AIR
        
        # process collisions
        collResp = collResponses["catReq"]
        # process x direction
        touchingRight = False
        touchingLeft = False
        for cr in collResp:
            if cr.intersects:
                if cr.collType == "leftwall":
                    touchingLeft = True
                    if touchingRight:
                        self.is_invincible = True
                        self.invincibleTimeLeft = pS.invincibilityTime
                        self.should_draw = True
                        self.draw_delta = 0
                        self.score -= 15
                        if self.anim != self.hurtAnim:
                                self.anim = self.hurtAnim
                                self.anim.reset()
                        break
                    (transX, transY) = coll.minTranslation(newRect, cr.collRect)
                    if transY == 0:
                        newX -= transX
                if cr.collType == "rightwall":
                    (transX, transY) = coll.minTranslation(newRect, cr.collRect)
                    if transY == 0:
                        newX -= transX
                if cr.collType == "block" and not self.is_invincible:
                    if newRect.Right >= cr.collRect.Left and self.rect.Bottom > cr.collRect.Top:
                        touchingRight = True
                        if touchingLeft:
                            self.is_invincible = True
                            self.invincibleTimeLeft = pS.invincibilityTime
                            self.should_draw = True
                            self.draw_delta = 0
                            self.score -= 15
                            if self.anim != self.hurtAnim:
                                self.anim = self.hurtAnim
                                self.anim.reset()
                            pass
                        (blockVel, isDamaging) = cr.misc
                        if isDamaging and not self.is_invincible:
                            self.is_invincible = True
                            self.invincibleTimeLeft = pS.invincibilityTime
                            self.should_draw = True
                            self.draw_delta = 0
                            self.score -= 10
                            if self.anim != self.hurtAnim:
                                self.anim = self.hurtAnim
                                self.anim.reset()
                            pass
                        (transX, transY) = coll.minTranslation(newRect, cr.collRect)
                        newX -= transX
                    elif newRect.Left <= cr.collRect.Right and self.rect.Bottom > cr.collRect.Top:
                        (blockVel, isDamaging) = cr.misc
                        if isDamaging and not self.is_invincible:
                            self.is_invincible = True
                            self.invincibleTimeLeft = pS.invincibilityTime
                            self.should_draw = True
                            self.draw_delta = 0
                            self.score -= 10
                            if self.anim != self.hurtAnim:
                                self.anim = self.hurtAnim
                                self.anim.reset()
                            pass
                        (transX, transY) = coll.minTranslation(newRect, cr.collRect)
                        newX -= transX

        for cr in collResp:
            if cr.intersects:
                if cr.collType == "floor":
                    self.state = pS.State.ON_GROUND
                    self.velY = 0;
                    (transX, transY) = coll.minTranslation(newRect, cr.collRect)
                    newY -= transY
                    self.jumped = False
                    self.can_jump = False
                if cr.collType == "block":
                    if newRect.Bottom >= cr.collRect.Top and self.rect.Bottom <= cr.collRect.Top:
                        if self.state == pS.State.ON_BLOCK:
                            pass
                        self.state = pS.State.ON_BLOCK
                        newY = cr.collRect.Top - (self.rect.Bottom - self.rect.Top)
                        self.velY = 0;
                        (blockVel, isDamaging) = cr.misc
                        newX += blockVel * g.timeDelta
                        self.jumped = False
                        self.can_jump = False
                        if isDamaging and not self.is_invincible:
                            self.is_invincible = True
                            self.invincibleTimeLeft = pS.invincibilityTime
                            self.should_draw = True
                            self.draw_delta = 0
                            self.score -= 10
                            if self.anim != self.hurtAnim:
                                self.anim = self.hurtAnim
                                self.anim.reset()
                    elif newRect.Top <= cr.collRect.Bottom and self.rect.Top >= cr.collRect.Bottom:
                        self.velY = 0
                        newY = cr.collRect.Bottom
                        (blockVel, isDamaging) = cr.misc
                        if isDamaging and not self.is_invincible:
                            self.is_invincible = True
                            self.invincibleTimeLeft = pS.invincibilityTime
                            self.should_draw = True
                            self.draw_delta = 0
                            self.score -= 10
                            if self.anim != self.hurtAnim:
                                self.anim = self.hurtAnim
                                self.anim.reset()
        if newY > pS.startY:
            newY = pS.startY
        self.move(newX, newY)
        if self.state == pS.State.ON_BLOCK and not self.is_invincible:
            self.score += g.timeDelta * 10
        if self.state == pS.State.ON_GROUND and not self.is_invincible:
            self.score -= g.timeDelta * 5
        if self.state == pS.State.IN_AIR and not self.is_invincible:
            self.score += g.timeDelta * 2.5
        if self.score < 0:
            self.score = 0.0
        self.updateAnimation()

    def unstickCollision(self, x, y, rect, collRect):
        (transX, transY) = coll.minTranslation(rect, collRect)
        if transY == 0.0:
            self.move(x - transX, self.y)
        else:
            self.move(self.x, y - transY)

    def updateAnimation(self):
        if self.anim == self.hurtAnim:
            if not self.hurtAnim.finished:
                return
        if self.state == pS.State.ON_GROUND or self.state == pS.State.ON_BLOCK:
            # idle
            if self.velX == 0.0:
                self.anim = self.idleAnim
            # walk
            else:
                self.anim = self.walkAnim
        else:
            # jump
            if self.velY < 0.0:
                if self.anim != self.jumpAnim:
                    self.anim = self.jumpAnim
                    self.anim.reset()
            # fall
            else:
                if self.anim != self.fallAnim:
                    self.anim = self.fallAnim
                    self.anim.reset()
            
def catTasklet():
    global cat
    cat = Cat(pS.startX, pS.startY)
    cat.goal = 100
    while 1:
        inputState = channels.inputChannel.receive()
        
        progress = 0
        if cat.score < cat.goal / 2:
            progress = 0
        elif cat.score < cat.goal - 15:
            progress = 1
        else:
            progress = 2
        bgChannels.progressionChannel.send(progress)
            
        cat.update(inputState)
        cat.draw()
        

def goalTasklet():
    global cat

    while 1:
        goal = bgChannels.goalChannel.receive()
        cat.reset(goal)
    
        


                
    
