import stackless as ss
from PySFML import sf
from settings.settings import *
import globals.globals as g
import channels.channels as channels
from . import channels as bgChannels
import graphics.graphics as graphics

class Progress:
    FORWARDS    = 0
    BACKWARDS   = 1
    STAY        = 2

layerStr = sf.String("Layer: ")
layerStr.SetColor(sf.Color.Black)
layerStr.SetPosition(20.0, 60.0)
layerStr.SetSize(15.0)

statusStr = sf.String("Status: ")
statusStr.SetColor(sf.Color.Black)
statusStr.SetPosition(20.0, 90.0)
statusStr.SetSize(15.0)

progressStr = sf.String("||")
progressStr.SetColor(sf.Color.Black)
progressStr.SetPosition(500.0, 40.0)
progressStr.SetSize(15.0)

music = []
progress = Progress.FORWARDS
musicIndex = 0
layers = 4
volume = 100.0

def drawStatus():
    global music
    global progress
    global musicIndex

    def progressToStr(progress):
        if progress == Progress.FORWARDS:
            return ">>"
        elif progress == Progress.BACKWARDS:
            return "<<"
        else:
            return "||"

    def timeFormat(time):
        return "%.2f" % time


    layerStr.SetText("Progression: " + str(musicIndex + 1))
    layerStr.SetPosition(100.0, 15.0)
    g.window.Draw(layerStr)

    playingOffset = music[musicIndex].GetPlayingOffset()
    duration = music[musicIndex].GetDuration()

    statusStr.SetPosition(850.0, 15.0)
    statusStr.SetText(timeFormat(playingOffset) + "/" +
                      timeFormat(duration))
    g.window.Draw(statusStr)

    x = 199.0
    y = 18.0
    tickLeft = sf.FloatRect(x, y, x + 1.0, y + 14.0)
    graphics.drawRect(g.window, tickLeft, sf.Color(100, 100, 100))

    x = 824.0
    y = 18.0
    tickRight = sf.FloatRect(x, y, x + 1.0, y + 14.0)
    graphics.drawRect(g.window, tickRight, sf.Color(100, 100, 100))

    x = 200.0
    y = 20.0
    maxWidth = 1024.0 - 2 * x

    width = (maxWidth / duration) * playingOffset

    playbackRect = sf.FloatRect(x, y, x + width, y + 10)
    graphics.drawRect(g.window, playbackRect, sf.Color.Black)

    progressStr.SetText(progressToStr(progress))
    g.window.Draw(progressStr)

def loadMusic():
    music = []
    for i in range(layers):
        music.append(sf.Music())
        music[i].OpenFromFile(dataPath + "music/song1/song1L" + str(i + 1) + ".ogg")
        music[i].Initialize(2, 44100)
        music[i].SetLoop(True)
        music[i].SetVolume(volume)

    return music

def musicTasklet():
    global music
    global progress
    global musicIndex

    music = loadMusic()
    musicIndex = 0
    music[musicIndex].Play()

    layerStr.SetText("Progression: " + str(musicIndex + 1))

    while 1:
        inputState = channels.inputChannel.receive()
        if inputState.keyPressed("f1"):
            progress = Progress.BACKWARDS
        elif inputState.keyPressed("f2"):
            progress = Progress.STAY
        elif inputState.keyPressed("f3"):
            progress = Progress.FORWARDS
        elif inputState.keyPressed("f4"):
            musicProgress()

        #catInfo = bgChannels.backgroundChannel.receive()
        #ss.schedule()

        if music[musicIndex].GetDuration() - music[musicIndex].GetPlayingOffset() <= 0.1:
            musicProgress()

        drawStatus()

def musicProgress():
    global music
    global progress
    global musicIndex

    if progress == Progress.FORWARDS:
        print("\tforwards!")
        newMusicIndex = musicIndex + 1
    elif progress == Progress.BACKWARDS:
        print("\tbackwards!")
        newMusicIndex = musicIndex - 1
    elif progress == Progress.STAY:
        #print("\tstay!")
        newMusicIndex = musicIndex

    if newMusicIndex < 0:
        newMusicIndex = 0
    elif newMusicIndex >= len(music):
        newMusicIndex = len(music) - 1

    if newMusicIndex == musicIndex:
        pass
    else:
        music[newMusicIndex].Play()
        music[musicIndex].Stop()

    musicIndex = newMusicIndex

    channels.broadcast(bgChannels.progressionChannel, newMusicIndex + 1)
